# Basic Hapi js API server using Hapi-mol

## What is Mol
Mol is a very simple Hapi plugin that allows you to easily batch load modules in your code.
Spread your code into separate, Point Mol the the location and tell it how to load it and your set.

## How to use it
Lets start setting up a basic Hapi server.
I'm using Hapi 8.0.0 API but Mol should works well with version 7 as well.
```javascript
var hapi   = require('hapi'),
	mol    = require('../'),
	server = new hapi.Server({ debug: { request: ['error'] } });

server.connection({port: 8000});
```
this is very straight forward.
we require our dependencies (Hapi, and Mol), then we create a server and
bind a connection to it on port 8000.

now we need to register Mol with Hapi
```javascript
server.register(mol, function (err){
	if (err) {
		throw err;
	}

	server.plugins.mol.load('controllers', function (path, base, ext, scope){
		var controller = require(path);
		server.route(controller);
	}).then(server.start.bind(server, function (err) {
		if (err) {
			console.error(err);
		} else {
			console.log('Server running at localhost:8000');
		}
	}));
});
```

### The load function
Mol exposes a load function on `server.plugins.mol.load`.
With this function we get access to all of the modules we have under a specific subfolder.
The first parameter we pass to the load function is the path of the subfolder in this case `./controllers`.
The second parameter is a scope parameter. The scope will be relative to subfolders so in our example when calling the register callback for './controllers/foo/bar.js' scope will be an Object under
`controllers.foo` and setting `scope[base]` will set `controllers.foo.bar`.
the last parameters we send to `load` is the register callback.

### The register callback
This is where most of the magic happens and where we actually load each file under our controller folder.
for each file the callback will be called and we will get a few parameters that will help us load our models.
the first parameter id the `path` parameter for the full path for the file we want to load.
for a simple logic all we need to do is loading this file with `require`.
if you look at one of our sample controllers you'll see that they export a simple array of Hapi routes so all we need is to register them on our server.

Thats it we nicely separated our app logic into separated file and loaded them.
Looks to simple ?
Even with this simple example we have improved our app structure dramatically
we can arrange our logic in separate files and folders loading each of them automatically and making sure our file structure and follows the same pattern for all the files.

But in the next post we can see how we can use Mol to make our structure and app scope even more robust and still keep our code simple and clean.